#include <rFlex.h>
#include <rFlexP.h>

/* ============================================================ */
/* ============================================================ */

static int
getSystemReq(struct reflex *reflex, unsigned char id,
	     unsigned char axis)
{
  int len;
  unsigned char buffer[256];
  unsigned char opcode;
  unsigned char port;
  int rv;

  len = 0;

  opcode = MOT_AXIS_GET_SYSTEM;
  port   = reflex->motion->port;
  pktAddChar(buffer, &len, axis);

  rv = pktWrite(reflex->pktPort, port, id, opcode, buffer, len);
  return (rv);
}

/*------------------------------------------------------------*/

static int
getSystemRpl(struct reflex *reflex, unsigned char id,
	     int len, unsigned char *buffer)
{
  int ii;
  int rv;
  int timeStamp;
  unsigned char axis;
  int pos;
  int vel;
  int acc;
  int trq;
  
  /* should check on len and opcode */

  ii = 0;

  rv = pktGetLong(buffer, &ii);
  timeStamp = pktGetLong(buffer, &ii);
  axis  = pktGetChar(buffer, &ii);
  pos   = pktGetLong(buffer, &ii);
  vel   = pktGetLong(buffer, &ii);
  acc   = pktGetLong(buffer, &ii);
  trq   = pktGetLong(buffer, &ii);

  if (reflex->motion->getSystemRpl) {
    rv = reflex->motion->getSystemRpl (reflex, rv, id, timeStamp,
					   axis, pos, vel, acc, trq);
  }
  return(rv);
}

/*------------------------------------------------------------*/

static int
systemRpt(struct reflex *reflex, unsigned char id,
	  int len, unsigned char *buffer)
{
  int ii;
  int rv;
  int timeStamp;
  unsigned char axis;
  int pos;
  int vel;
  int acc;
  int trq;
  
  /* should check on len and opcode */

  ii = 0;

  rv = pktGetLong(buffer, &ii);
  timeStamp = pktGetLong(buffer, &ii);
  axis  = pktGetChar(buffer, &ii);
  pos   = pktGetLong(buffer, &ii);
  vel   = pktGetLong(buffer, &ii);
  acc   = pktGetLong(buffer, &ii);
  trq   = pktGetLong(buffer, &ii);

  if (reflex->motion->systemRpt) {
    rv = reflex->motion->systemRpt (reflex, id, timeStamp,
				    axis, pos, vel, acc, trq);
  }
  return(rv);
}

/*------------------------------------------------------------*/

static int
getModelReq(struct reflex *reflex, unsigned char id,
	    unsigned char axis)
{
  int len;
  unsigned char buffer[256];
  unsigned char opcode;
  unsigned char port;
  int rv;

  len = 0;

  opcode = MOT_AXIS_GET_MODEL;
  port   = reflex->motion->port;
  pktAddChar(buffer, &len, axis);

  rv = pktWrite(reflex->pktPort, port, id, opcode, buffer, len);
  return (rv);
}

/*------------------------------------------------------------*/

static int
getModelRpl(struct reflex *reflex, unsigned char id,
	    int len, unsigned char *buffer)
{
  int ii;
  int rv;
  int timeStamp;
  unsigned char axis;
  int pos;
  int vel;
  int acc;
  int trq;
  
  /* should check on len and opcode */

  ii = 0;

  rv = pktGetLong(buffer, &ii);
  timeStamp = pktGetLong(buffer, &ii);
  axis  = pktGetChar(buffer, &ii);
  pos   = pktGetLong(buffer, &ii);
  vel   = pktGetLong(buffer, &ii);
  acc   = pktGetLong(buffer, &ii);
  trq   = pktGetLong(buffer, &ii);

  if (reflex->motion->getModelRpl) {
    rv = reflex->motion->getModelRpl (reflex, rv, id, timeStamp,
					  axis, pos, vel, acc, trq);
  }
  return(rv);
}

/*------------------------------------------------------------*/

static int
getTargetReq(struct reflex *reflex, unsigned char id,
	     unsigned char axis)
{
  int len;
  unsigned char buffer[256];
  unsigned char opcode;
  unsigned char port;
  int rv;

  len = 0;

  opcode = MOT_AXIS_GET_TARGET;
  port   = reflex->motion->port;
  pktAddChar(buffer, &len, axis);

  rv = pktWrite(reflex->pktPort, port, id, opcode, buffer, len);
  return (rv);
}

/*------------------------------------------------------------*/

static int
getTargetRpl(struct reflex *reflex, unsigned char id,
	     int len, unsigned char *buffer)
{
  int ii;
  int rv;
  int timeStamp;
  unsigned char axis;
  int pos;
  int vel;
  int acc;
  int trq;
  
  /* should check on len and opcode */

  ii = 0;

  rv = pktGetLong(buffer, &ii);
  timeStamp = pktGetLong(buffer, &ii);
  axis  = pktGetChar(buffer, &ii);
  pos   = pktGetLong(buffer, &ii);
  vel   = pktGetLong(buffer, &ii);
  acc   = pktGetLong(buffer, &ii);
  trq   = pktGetLong(buffer, &ii);

  if (reflex->motion->getTargetRpl) {
    rv = reflex->motion->getTargetRpl (reflex, rv, id, timeStamp,
					   axis, pos, vel, acc, trq);
  }
  return(rv);
}

/*------------------------------------------------------------*/

static int
setDirReq       (struct reflex *reflex, unsigned char id,
		 unsigned char axis,
		 int vel, int acc, int trq, unsigned char dir)
{
  int len;
  unsigned char buffer[256];
  unsigned char opcode;
  unsigned char port;
  int rv;

  len = 0;

  opcode = MOT_AXIS_SET_DIR;
  port   = reflex->motion->port;

  pktAddChar(buffer, &len, axis);
  pktAddLong(buffer, &len, vel);
  pktAddLong(buffer, &len, acc);
  pktAddLong(buffer, &len, trq);
  pktAddChar(buffer, &len, dir);

  rv = pktWrite(reflex->pktPort, port, id, opcode, buffer, len);
  return (rv);
}

/*------------------------------------------------------------*/

static int
setDirRpl (struct reflex *reflex, unsigned char id,
	   int len, unsigned char *buffer)
{
  int ii;
  int rv;
  int timeStamp;
  unsigned char axis;

  /* should check on len and opcode */

  ii = 0;

  rv = pktGetLong(buffer, &ii);
  timeStamp = pktGetLong(buffer, &ii);
  axis   = pktGetChar(buffer, &ii);
  
  if (reflex->motion->setDirRpl) {
    rv = reflex->motion->setDirRpl (reflex, rv, id, timeStamp, axis);
  }
  return(rv);
}

/*------------------------------------------------------------*/

static int
setPosReq       (struct reflex *reflex, unsigned char id,
		 unsigned char axis,
		 int vel, int acc, int trq, int pos)
{
  int len;
  unsigned char buffer[256];
  unsigned char opcode;
  unsigned char port;
  int rv;

  len = 0;

  opcode = MOT_AXIS_SET_POS;
  port   = reflex->motion->port;

  pktAddChar(buffer, &len, axis);
  pktAddLong(buffer, &len, vel);
  pktAddLong(buffer, &len, acc);
  pktAddLong(buffer, &len, trq);
  pktAddLong(buffer, &len, pos);

  rv = pktWrite(reflex->pktPort, port, id, opcode, buffer, len);
  return (rv);
}

/*------------------------------------------------------------*/

static int
setPosRpl (struct reflex *reflex, unsigned char id,
	   int len, unsigned char *buffer)
{
  int ii;
  int rv;
  int timeStamp;
  unsigned char axis;

  /* should check on len and opcode */

  ii = 0;

  rv = pktGetLong(buffer, &ii);
  timeStamp = pktGetLong(buffer, &ii);
  axis   = pktGetChar(buffer, &ii);
  
  if (reflex->motion->setDirRpl) {
    rv = reflex->motion->setDirRpl (reflex, rv, id, timeStamp, axis);
  }
  return(rv);
}

/*------------------------------------------------------------*/

static int
setDefaultsReq  (struct reflex *reflex, unsigned char id)
{
  int len;
  unsigned char buffer[256];
  unsigned char opcode;
  unsigned char port;
  int rv;

  len = 0;

  opcode = MOT_SET_DEFAULTS;
  port   = reflex->motion->port;

  rv = pktWrite(reflex->pktPort, port, id, opcode, buffer, len);
  return (rv);
}

/*------------------------------------------------------------*/

static int
setDefaultsRpl  (struct reflex *reflex, unsigned char id, int len, unsigned char *buffer)
{
  int ii;
  int rv;
  int timeStamp;

  /* should check on len and opcode */

  ii = 0;

  rv = pktGetLong(buffer, &ii);
  timeStamp = pktGetLong(buffer, &ii);

  if (reflex->motion->setDefaultsRpl) {
    rv =  reflex->motion->setDefaultsRpl(reflex, rv, id, timeStamp);
  }
  return(rv);
}

/*------------------------------------------------------------*/

static int
setSystemRptReq(struct reflex *reflex, unsigned char id,
		unsigned long period, unsigned long mask)
{
  int len;
  unsigned char buffer[256];
  unsigned char opcode;
  unsigned char port;
  int rv;

  len = 0;

  opcode = MOT_SYSTEM_REPORT_REQ;
  port   = reflex->motion->port;
  pktAddLong(buffer, &len, period);
  pktAddLong(buffer, &len, mask);

  rv = pktWrite(reflex->pktPort, port, id, opcode, buffer, len);
  return (rv);
}

/*------------------------------------------------------------*/

static int
setSystemRptRpl (struct reflex *reflex, unsigned char id,
		 int len, unsigned char *buffer)
{
  int ii;
  int rv;
  int timeStamp;

  /* should check on len and opcode */

  ii = 0;

  rv = pktGetLong(buffer, &ii);
  timeStamp = pktGetLong(buffer, &ii);

  if (reflex->motion->setSystemRptRpl) {
    rv =  reflex->motion->setSystemRptRpl(reflex, rv, id, timeStamp);
  }
  return(rv);
}

/*------------------------------------------------------------*/

static int
brakeSetReq  (struct reflex *reflex, unsigned char id)
{
  int len;
  unsigned char buffer[256];
  unsigned char opcode;
  unsigned char port;
  int rv;

  len = 0;

  opcode = MOT_BRAKE_SET;
  port   = reflex->motion->port;

  rv = pktWrite(reflex->pktPort, port, id, opcode, buffer, len);
  return (rv);
}

/*------------------------------------------------------------*/

static int
brakeSetRpl (struct reflex *reflex, unsigned char id, int len, unsigned char *buffer)
{
  int ii;
  int rv;
  int timeStamp;

  /* should check on len and opcode */

  ii = 0;

  rv = pktGetLong(buffer, &ii);
  timeStamp = pktGetLong(buffer, &ii);
  
  if (reflex->motion->brakeSetRpl) {
    rv = reflex->motion->brakeSetRpl(reflex, rv, id, timeStamp);
  }
  return(rv);
}

/* ============================================================ */

static int
brakeReleaseReq(struct reflex *reflex, unsigned char id)
{
  int len;
  unsigned char buffer[256];
  unsigned char opcode;
  unsigned char port;
  int rv;

  len = 0;

  opcode = MOT_BRAKE_RELEASE;
  port   = reflex->motion->port;

  rv = pktWrite(reflex->pktPort, port, id, opcode, buffer, len);
  return (rv);
}

/*------------------------------------------------------------*/

static int
brakeReleaseRpl(struct reflex *reflex, unsigned char id, int len, unsigned char *buffer)
{
  int ii;
  int rv;
  int timeStamp;

  /* should check on len and opcode */

  ii = 0;

  rv     = pktGetLong(buffer, &ii);
  timeStamp  = pktGetLong(buffer, &ii);

  if (reflex->motion->brakeReleaseRpl) {
    rv = reflex->motion->brakeReleaseRpl(reflex, rv, id, timeStamp);
  }
  return(rv);
}

/* ============================================================ */
/* ============================================================ */
/* ============================================================ */

static int
handlePort(struct reflex *reflex, int size, unsigned char *buffer,
	   unsigned char id, unsigned char opcode) {
  int rv;

  switch (opcode) {
  case MOT_AXIS_GET_SYSTEM:
    rv = getSystemRpl(reflex, id, size, buffer);
    break;

  case MOT_SYSTEM_REPORT:
    rv = systemRpt(reflex, id, size, buffer);
    break;

  case MOT_AXIS_GET_MODEL:
    rv = getModelRpl(reflex, id, size, buffer);
    break;

  case MOT_AXIS_GET_TARGET:
    rv = getTargetRpl(reflex, id, size, buffer);
    break;

  case MOT_AXIS_SET_DIR:
    rv = setDirRpl(reflex, id, size, buffer);
    break;

  case MOT_SET_DEFAULTS:
    rv = setDefaultsRpl(reflex, id, size, buffer);
    break;

  case MOT_SYSTEM_REPORT_REQ:
    rv = setSystemRptRpl(reflex, id, size, buffer);
    break;

  case MOT_BRAKE_SET:
    rv = brakeSetRpl(reflex, id, size, buffer);
    break;

  case MOT_BRAKE_RELEASE:
    rv = brakeReleaseRpl(reflex, id, size, buffer);
    break;

  default:
    rv = 0;
    if (reflex->motion->unknownRpl) {
      rv = reflex->motion->unknownRpl(reflex, id, size, buffer, opcode);
    }
  }

  return(rv);
}

/* ============================================================ */
/* ============================================================ */

static int
destroy (struct reflexMotion *motion) {

  if (motion) free(motion);
  return(0);
}

/* ------------------------------------------------------------ */

struct reflexMotion *
reflexMotionNew (int port) {
  struct reflexMotion *motion;

  motion = malloc(sizeof(*motion));
  if (!motion) return(NULL);
  memset(motion, 0, sizeof(*motion));

  motion->port       = port;

  motion->getSystemReq    = getSystemReq;
  motion->getModelReq     = getModelReq;
  motion->getTargetReq    = getTargetReq;
  motion->setDirReq       = setDirReq;
  motion->setPosReq       = setPosReq;
  motion->setDefaultsReq  = setDefaultsReq;
  motion->setSystemRptReq = setSystemRptReq;
  motion->brakeSetReq     = brakeSetReq;
  motion->brakeReleaseReq = brakeReleaseReq;

  motion->handlePort = handlePort;
  motion->destroy    = destroy;

  return(motion);
}
