#include <rFlex.h>
#include <rFlexP.h>

/* ============================================================ */
/* ============================================================ */

static int
updateReq(struct reflex *reflex, unsigned char id, int timeStamp)
{
  int len;
  unsigned char buffer[256];
  unsigned char opcode;
  unsigned char port;
  int retval;

  len = 0;

  opcode = SONAR_GET_UPDATE;
  port   = reflex->sonar->port;

  pktAddLong(buffer, &len, timeStamp);

  retval = pktWrite(reflex->pktPort, port, id, opcode, buffer, len);
  return (retval);
}
  
/*------------------------------------------------------------*/

static int
updateRpl(struct reflex *reflex, unsigned char id,
	  int len, unsigned char *buffer)
{
  int ii;
  int retval;
  int count;
  int timeStamp;
  unsigned char    sid[256];
  unsigned short value[256];

  /* should check on len and opcode */

  ii = 0;
  count = 0;

  retval = pktGetLong(buffer, &ii);
  timeStamp = pktGetLong(buffer, &ii);

  while ((ii<len) && (count<256)) {
    sid[count]   = pktGetChar(buffer, &ii);
    value[count] = pktGetShort(buffer, &ii);
    count++;
  }

  if (reflex->sonar->updateRpl) {
    retval = reflex->sonar->updateRpl(reflex, retval, id, timeStamp,
				      count, sid, value);
  }

  return(retval);
}
  
/*------------------------------------------------------------*/

static int
sonarRpt(struct reflex *reflex, unsigned char id,
	 int len, unsigned char *buffer)
{
  int ii;
  int retval;
  int count;
  int timeStamp;
  unsigned char    sid[256];
  unsigned short value[256];

  /* should check on len and opcode */

  ii = 0;
  count = 0;

  retval = pktGetLong(buffer, &ii);
  timeStamp = pktGetLong(buffer, &ii);

  while ((ii<len) && (count<256)) {
    sid[count]   = pktGetChar(buffer, &ii);
    value[count] = pktGetShort(buffer, &ii);
    count++;
  }

  if (reflex->sonar->rpt) {
    retval = reflex->sonar->rpt(reflex, id, timeStamp,
				count, sid, value);
  }

  return(retval);
}
  
/*------------------------------------------------------------*/

static int
rptReq(struct reflex *reflex, unsigned char id,
       int echoDelay, int pingDelay, int setDelay, unsigned char mode)
{
  int len;
  unsigned char buffer[256];
  unsigned char opcode;
  unsigned char port;
  int retval;

  len = 0;

  opcode = SONAR_RUN;
  port   = reflex->sonar->port;

  pktAddLong(buffer, &len, echoDelay);
  pktAddLong(buffer, &len, pingDelay);
  pktAddLong(buffer, &len, setDelay);
  pktAddChar(buffer, &len, mode);

  retval = pktWrite(reflex->pktPort, port, id, opcode, buffer, len);
  return (retval);
}
  
/*------------------------------------------------------------*/

static int
rptRpl (struct reflex* reflex, unsigned char id,
	int len, unsigned char *buffer)
{
  int ii;
  int retval;
  int timeStamp;

  /* should check on len and opcode */

  ii = 0;

  retval    = pktGetLong(buffer, &ii);
  timeStamp = pktGetLong(buffer, &ii);

  if (reflex->sonar->rptRpl) {
    retval = reflex->sonar->rptRpl(reflex, retval, id, timeStamp);
  }
  return(retval);
}

/* ============================================================ */
/* ============================================================ */
/* ============================================================ */

static int
handlePort(struct reflex* reflex,
	   int size, unsigned char *buffer,
	   unsigned char id, unsigned char opcode)
{
  int retval;

  switch (opcode) {
  case SONAR_RUN:
    retval = rptRpl       (reflex, id, size, buffer);
    break;

  case SONAR_GET_UPDATE:
    retval = updateRpl    (reflex, id, size, buffer);
    break;

  case SONAR_REPORT:
    retval = sonarRpt     (reflex, id, size, buffer);
    break;

  default:
    retval = 0;
    if (reflex->sonar->unknownRpl) {
      retval = reflex->sonar->unknownRpl(reflex, id, size, buffer, opcode);
    }
  }

  return(retval);
}

/* ============================================================ */
/* ============================================================ */

static int
destroy (struct reflexSonar *sonar) {

  if (sonar) free(sonar);
  return(0);
}

/* ------------------------------------------------------------ */

struct reflexSonar *
reflexSonarNew (int port) {
  struct reflexSonar *sonar;

  sonar = malloc(sizeof(*sonar));
  if (!sonar) return(NULL);
  memset(sonar, 0, sizeof(*sonar));

  sonar->port       = port;

  sonar->updateReq  = updateReq;
  sonar->rptReq     = rptReq;

  sonar->handlePort = handlePort;
  sonar->destroy    = destroy;

  return(sonar);
}
